'''
    twiz - Your Twitter Account Data Analysis & Visualization Tool <3

    twiz can help you in analyzing and visualizing your twitter account's
    exported data offline. 

    Author: Anjan Roy <anjanroy@yandex.com>
    License: CC0-1.0
'''
__version__ = '0.3.0'

from warnings import filterwarnings

filterwarnings('ignore')
