#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['twiz',
 'twiz.model',
 'twiz.model.ad',
 'twiz.model.ad.engagement',
 'twiz.plot',
 'twiz.plot.ad',
 'twiz.plot.ad.engagement']

package_data = \
{'': ['*']}

install_requires = \
['matplotlib', 'seaborn', 'emoji']

extras_require = \
{'dev': ['flit']}

entry_points = \
{'console_scripts': ['twiz = twiz.main:main']}

setup(name='twiz',
      version='0.3.0',
      description='twiz - Your Twitter Account Data Analysis & Visualization Tool <3',
      author='Anjan Roy',
      author_email='anjanroy@yandex.com',
      url='https://itzmeanjan.github.io/twiz',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.7',
     )
